;----------------------------------------------------------------------------
;
;       JTT Memory Management System
;       ----------------------------
;
;       (C)1998 by Jari Tapio Tuominen, all rights reserved.
;
;
;       Description:
;       
;       This library is designed to help memory allocating,
;       it is especially helpful when more than one program is
;       using same memory.
;
;
;       MALLOC
;
;       malloc( [size in blocks] );
;
;       Returns: Memory ID
;                or 255 if out memory.
;
;       
;       FREE
;       
;       free( memoryid );
;       
;       Returns: Nothing
;
;
;----------------------------------------------------------------------------
jbamfree        lda r0
                ldx r1
                jmp bamfree
;----------------------------------------------------------------------------
jbamalloc       lda r0
                ldx r1
                jmp bamalloc
;----------------------------------------------------------------------------
;JMALLOC
;Arguments:
;       r0=Number of blocks to allocates
;Returns:
;       r0=Memory ID
;       r1=Offset to memory
jmalloc         lda r0
                jsr malloc
                sta r0
                stx r1
                rts
;----------------------------------------------------------------------------
jfree           lda r0
                jmp free
;----------------------------------------------------------------------------
memreset        lda #0
                ldx #255
                jsr bamfree
                lda #255
                sta memtaba
                sta memtabb
                rts
;----------------------------------------------------------------------------
free            pha
                ;
                tay
                lda memtaba,y
                clc
                adc memtabb,y
                tax
                lda memtaba,y
                jsr bamfree
                ;
                pla
                lda #$fe
                sta memtaba,y
                sta memtabb,y
                ;
                rts
;----------------------------------------------------------------------------
;       MALLOC  **  Allows: DYMAL, Dynamic Memory Allocating.
;       ------
;
;Description:
;
;     MALLOC function and other related functions
;     are required to prove completely working
;     Dynamic Memory Allocation System with minimized options.
;     Every Operating System in todays OS markets owns
;     somekind of ability for Dynamic Memory Allocating.
;     
;
;Arguments:
;       a=Number of blocks to allocate
;
;Returns:
;
;       a=Memory KEY (Needed when modifying allocated memory)
;       ** a=255 if error.
;       x=MSB based offset to memory region ($10xx).
;
;
malloc          sta bamlsize
                ; Start searching for free memory region
                lda #0
                sta bamx
malloc_l1       lda bamx
                jsr bamget
                beq malloc_l2
                inc bamx
                beq malloc_error
                jmp malloc_l1
malloc_l2       lda bamx
                pha
                lda bamlsize
                sta bamy
                ; Confirm that free region is enough big.
malloc_l3       lda bamy
                beq malloc_l5
                lda bamx
                jsr bamget
                beq malloc_l4
                pla
                tax
                inx
                stx bamx
                jmp malloc_l1
malloc_l4       inc bamx
                dec bamy
                jmp malloc_l3

                ; Found free space at offset BAMX
malloc_l5       pla ; Pull stack into original position,
                    ; just to prevent program runtime error(s).
                lda bamx
                sta bama3
                ;
                lda bama3
                clc
                adc bamlsize
                tax
                lda bama3
                jsr bamalloc
                ;
                lda bama3
                ldx bamlsize
                jsr memtabinsert
                ldx bama3
                rts
                ;
malloc_error    lda #255
                rts
;----------------------------------------------------------------------------
memtabinsert    pha
                ldy #0
memtabinsert_l1 lda memtaba,y
                bpl memtabinsert_l2
                lda memtabb,y
                bpl memtabinsert_l2
                pla
                sta memtaba,y
                txa
                sta memtabb,y
                tya
                rts
memtabinsert_l2 iny
                bne memtabinsert_l1
                rts
;----------------------------------------------------------------------------
bamgetfree      lda #0
                sta bamx
                sta bamy
bamgetfree_l1   lda bamx
                jsr bamget
                bne bamgetfree_l2
                inc bamy
bamgetfree_l2   inc bamx
                bne bamgetfree_l1
                lda bamy
                rts
;----------------------------------------------------------------------------
bamset          jsr bamcalc
                lda bambits,x
                ora membam,y
                sta membam,x
                rts
;----------------------------------------------------------------------------
bamunset        jsr bamcalc
                lda bambits,x
                eor #$ff
                sta bama1
                lda membam,y
                and bama1
                sta membam,y
                rts                
;----------------------------------------------------------------------------
bamget          jsr bamcalc
                lda membam,y
                and bambits,x
                rts
;----------------------------------------------------------------------------
bamcalc         pha
                lsr
                lsr
                lsr
                tay
                pla
                and #7
                tax
                rts
;----------------------------------------------------------------------------
; Allocates membam from a to x
bamalloc        sta bamx
                stx bamy
bamalloc_l1     ldx bamx
                dex
                cpx bamy
                beq bamalloc_l2
                lda bamx
                jsr bamset
                inc bamx
                jmp bamalloc_l1
bamalloc_l2     rts
;----------------------------------------------------------------------------
; Frees membam from a to x
bamfree         sta bamx
                stx bamy
bamfree_l1      ldx bamx
                dex
                cpx bamy
                beq bamfree_l2
                lda bamx
                jsr bamunset
                inc bamx
                jmp bamfree_l1
bamfree_l2      rts
;----------------------------------------------------------------------------
bambits         .byte 128,64,32,16,8,4,2,1
bama1           .byte 0
bama2           .byte 0
bama3           .byte 0
bamx            .byte 0
bamy            .byte 0
bamlsize        .byte 0 ; Last allocation size in blocks
;----------------------------------------------------------------------------
; 1. Offset  2. Length
; (In blocks)
;
; $fe $fe -> Freed memory
; $ff $ff -> End of MEMTABx
; ..rest..
; $xx $xx -> Allocated memory
;
;
; Space for 16 memory regions allocating :
memtaba         .word 255,255,255,255,255,255,255,255   ; OFFSETS
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255   ; OFFSETS
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
memtabb         .word 255,255,255,255,255,255,255,255   ; LENGTHS
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255   ; LENGTHS
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255
                .word 255,255,255,255,255,255,255,255

; 1bit means one block in memory, 32words/64bytes = 256 bits.
membam          .word 0,0,0,0,0,0,0,0
                .word 0,0,0,0,0,0,0,0
                .word 0,0,0,0,0,0,0,0
                .word 0,0,0,0,0,0,0,0


